"""Python wrapper for getting data from Brother laser and inkjet printers via SNMP."""
from __future__ import annotations

import logging
import re
from collections.abc import Generator, Iterable
from contextlib import suppress
from datetime import datetime, timedelta, timezone
from typing import TYPE_CHECKING, Any, cast

import pysnmp.hlapi.asyncio as hlapi
from dacite import from_dict
from pysnmp.error import PySnmpError
from pysnmp.hlapi.asyncio.cmdgen import lcd
from pysnmp.smi.rfc1902 import ObjectType

from .const import (
    ATTR_CHARSET,
    ATTR_COUNTERS,
    ATTR_FIRMWARE,
    ATTR_MAINTENANCE,
    ATTR_MODEL,
    ATTR_NEXTCARE,
    ATTR_PAGE_COUNT,
    ATTR_SERIAL,
    ATTR_STATUS,
    ATTR_UPTIME,
    CHARSET_MAP,
    OIDS,
    OIDS_HEX,
    PERCENT_VALUES,
    PRINTER_TYPES,
    UNSUPPORTED_MODELS,
    UPTIME_FLUCTUATION_SECONDS,
    VALUES_COUNTERS,
    VALUES_INK_MAINTENANCE,
    VALUES_LASER_MAINTENANCE,
    VALUES_LASER_NEXTCARE,
)
from .exceptions import SnmpError, UnsupportedModelError
from .model import BrotherSensors

_LOGGER = logging.getLogger(__name__)

REGEX_MODEL_PATTERN = re.compile(r"MDL:(?P<model>[\w\-]+)")


class Brother:
    """Main class to perform snmp requests to printer."""

    def __init__(
        self,
        host: str,
        port: int = 161,
        printer_type: str = "laser",
        model: str | None = None,
        snmp_engine: hlapi.SnmpEngine = None,
    ) -> None:
        """Initialize."""
        if model:
            _LOGGER.debug("Model: %s", model)
            for unsupported_model in UNSUPPORTED_MODELS:
                if unsupported_model in model.lower():
                    raise UnsupportedModelError(
                        "It seems that this printer model is not supported"
                    )

        if printer_type not in PRINTER_TYPES:
            _LOGGER.warning("Wrong printer_type argument, 'laser' was used")
            self._printer_type = "laser"
        else:
            self._printer_type = printer_type

        self._legacy = False

        self.firmware: str | None = None
        self.model: str | None = None
        self.serial: str
        self._host = host
        self._port = port
        self._last_uptime: datetime | None = None
        self._snmp_engine = snmp_engine
        self._oids: list[ObjectType] = []

    @classmethod
    async def create(
        cls,
        host: str,
        port: int = 161,
        printer_type: str = "laser",
        model: str | None = None,
        snmp_engine: hlapi.SnmpEngine = None,
    ) -> Brother:
        """Create a new device instance."""
        instance = cls(host, port, printer_type, model, snmp_engine)
        await instance.initialize()
        return instance

    async def initialize(self) -> None:
        """Initialize snmp_engine and check which OIDs are supported."""
        _LOGGER.debug("Initializing device %s", self._host)

        if not self._snmp_engine:
            self._snmp_engine = hlapi.SnmpEngine()

        oids = list(self._iterate_oids(OIDS.values()))

        try:
            request_args = [
                self._snmp_engine,
                hlapi.CommunityData("public", mpModel=0),
                hlapi.UdpTransportTarget(
                    (self._host, self._port), timeout=2, retries=10
                ),
                hlapi.ContextData(),
            ]
        except PySnmpError as err:
            raise ConnectionError(err) from err

        while True:
            get_result = await hlapi.getCmd(*request_args, *oids)
            _, errstatus, errindex, _ = get_result

            if str(errstatus) == "noSuchName":
                # 5 and 8 are indexes from OIDS consts, model and serial are obligatory
                if errindex in (5, 8):
                    raise UnsupportedModelError(
                        "It seems that this printer model is not supported"
                    )

                oids.pop(errindex - 1)
                continue

            break

        self._oids = oids

    async def async_update(self) -> BrotherSensors:
        """Update data from printer."""
        if not (raw_data := await self._get_data()):
            raise SnmpError("The printer did not return data")

        _LOGGER.debug("RAW data: %s", raw_data)

        data: dict[str, str | int | datetime | None] = {}

        try:
            model_match = re.search(REGEX_MODEL_PATTERN, raw_data[OIDS[ATTR_MODEL]])
            if TYPE_CHECKING:
                assert model_match is not None  # noqa: S101
            self.model = data[ATTR_MODEL] = cast(str, model_match.group("model"))
            self.serial = data[ATTR_SERIAL] = raw_data[OIDS[ATTR_SERIAL]]
        except (TypeError, AttributeError, AssertionError) as err:
            raise UnsupportedModelError(
                "It seems that this printer model is not supported"
            ) from err

        self.firmware = data[ATTR_FIRMWARE] = raw_data.get(OIDS[ATTR_FIRMWARE])

        charset = CHARSET_MAP.get(raw_data.get(OIDS[ATTR_CHARSET], "unknown"), "roman8")

        if status := raw_data[OIDS[ATTR_STATUS]]:
            data[ATTR_STATUS] = (
                self._cleanse_status(status).encode("latin1").decode(charset).lower()
            )

        try:
            uptime = int(cast(str, raw_data.get(OIDS[ATTR_UPTIME]))) / 100
        except TypeError:
            pass
        else:
            if self._last_uptime:
                new_uptime = (
                    datetime.now(tz=timezone.utc) - timedelta(seconds=uptime)
                ).replace(microsecond=0, tzinfo=timezone.utc)
                if (
                    abs((new_uptime - self._last_uptime).total_seconds())
                    > UPTIME_FLUCTUATION_SECONDS
                ):
                    data[ATTR_UPTIME] = self._last_uptime = new_uptime
                else:
                    data[ATTR_UPTIME] = self._last_uptime
            else:
                data[ATTR_UPTIME] = self._last_uptime = (
                    datetime.now(tz=timezone.utc) - timedelta(seconds=uptime)
                ).replace(microsecond=0, tzinfo=timezone.utc)
        if self._legacy:
            if self._printer_type == "laser":
                data.update(
                    dict(
                        self._iterate_data_legacy(
                            raw_data.get(OIDS[ATTR_MAINTENANCE], {}),
                            VALUES_LASER_MAINTENANCE,
                        )
                    )
                )
            if self._printer_type == "ink":
                data.update(
                    dict(
                        self._iterate_data_legacy(
                            raw_data.get(OIDS[ATTR_MAINTENANCE], {}),
                            VALUES_INK_MAINTENANCE,
                        )
                    )
                )
        else:
            if self._printer_type == "laser":
                data.update(
                    dict(
                        self._iterate_data(
                            raw_data.get(OIDS[ATTR_COUNTERS], {}), VALUES_COUNTERS
                        )
                    )
                )
                data.update(
                    dict(
                        self._iterate_data(
                            raw_data.get(OIDS[ATTR_MAINTENANCE], {}),
                            VALUES_LASER_MAINTENANCE,
                        )
                    )
                )
                data.update(
                    dict(
                        self._iterate_data(
                            raw_data.get(OIDS[ATTR_NEXTCARE], {}), VALUES_LASER_NEXTCARE
                        )
                    )
                )
            if self._printer_type == "ink":
                data.update(
                    dict(
                        self._iterate_data(
                            raw_data.get(OIDS[ATTR_COUNTERS], {}), VALUES_COUNTERS
                        )
                    )
                )
                data.update(
                    dict(
                        self._iterate_data(
                            raw_data.get(OIDS[ATTR_MAINTENANCE], {}),
                            VALUES_INK_MAINTENANCE,
                        )
                    )
                )
        # page counter for old printer models
        with suppress(ValueError):
            if not data.get(ATTR_PAGE_COUNT) and raw_data.get(OIDS[ATTR_PAGE_COUNT]):
                data[ATTR_PAGE_COUNT] = int(
                    cast(str, raw_data.get(OIDS[ATTR_PAGE_COUNT]))
                )

        _LOGGER.debug("Data: %s", data)

        result: BrotherSensors = from_dict(BrotherSensors, data)

        return result

    def shutdown(self) -> None:
        """Unconfigure SNMP engine."""
        if self._snmp_engine:
            lcd.unconfigure(self._snmp_engine, None)

    async def _get_data(self) -> dict[str, Any]:
        """Retrieve data from printer."""
        raw_data = {}

        try:
            request_args = [
                self._snmp_engine,
                hlapi.CommunityData("public", mpModel=0),
                hlapi.UdpTransportTarget(
                    (self._host, self._port), timeout=2, retries=10
                ),
                hlapi.ContextData(),
            ]
            get_result = await hlapi.getCmd(*request_args, *self._oids)
            errindication, errstatus, errindex, restable = get_result
        except PySnmpError as err:
            raise ConnectionError(err) from err
        if errindication:
            raise SnmpError(errindication)
        if errstatus:
            raise SnmpError(f"{errstatus}, {errindex}")
        for resrow in restable:
            if str(resrow[0]) in OIDS_HEX:
                # asOctets gives bytes data b'c\x01\x04\x00\x00\x00\x01\x11\x01\x04\x00\
                # x00\x05,A\x01\x04\x00\x00"\xc41\x01\x04\x00\x00\x00\x01o\x01\x04\x00\
                # x00\x19\x00\x81\x01\x04\x00\x00\x00F\x86\x01\x04\x00\x00\x00\n\xff'
                temp = resrow[-1].asOctets()
                # convert to string without checksum FF at the end, gives
                # '630104000000011101040000052c410104000022c4310104000000016f01040000190
                #  0810104000000468601040000000a'
                temp = "".join([f"{x:02x}" for x in temp])[0:-2]
                # split to 14 digits words in list, gives ['63010400000001',
                # '1101040000052c', '410104000022c4', '31010400000001',
                # '6f010400001900', '81010400000046', '8601040000000a']
                temp = [temp[ind : ind + 14] for ind in range(0, len(temp), 14)]
                # map sensors names to OIDs
                raw_data[str(resrow[0])] = temp
            else:
                raw_data[str(resrow[0])] = str(resrow[-1])
        # for legacy printers
        for resrow in restable:
            if str(resrow[0]) == OIDS[ATTR_MAINTENANCE]:
                # asOctets() gives bytes data
                temp = resrow[-1].asOctets()
                # convert to string without checksum FF at the end, gives
                # 'a101020414a201020c14a301020614a401020b14'
                temp = "".join([f"{x:02x}" for x in temp])[0:-2]
                if self._legacy_printer(temp):
                    self._legacy = True
                    # split to 10 digits words in list, gives ['a101020414',
                    # 'a201020c14', 'a301020614', 'a401020b14']
                    temp = [temp[ind : ind + 10] for ind in range(0, len(temp), 10)]
                    # map sensors names to OIDs
                    raw_data[str(resrow[0])] = temp
                    break
        return raw_data

    @staticmethod
    def _legacy_printer(string: str) -> bool:
        """Return True if printer is legacy."""
        length = len(string)
        nums = [x * 10 for x in range(length // 10)][1:]
        if results := [string[i - 2 : i] == "14" for i in nums]:
            return all(item for item in results)
        return False

    @staticmethod
    def _iterate_oids(oids: Iterable) -> Generator:
        """Iterate OIDS to retrieve from printer."""
        for oid in oids:
            yield hlapi.ObjectType(hlapi.ObjectIdentity(oid))

    @staticmethod
    def _iterate_data(iterable: Iterable, values_map: dict[str, str]) -> Generator:
        """Iterate data from hex words."""
        for item in iterable:
            # first byte means kind of sensor, last 4 bytes means value
            if item[:2] in values_map:
                if values_map[item[:2]] in PERCENT_VALUES:
                    yield (values_map[item[:2]], round(int(item[-8:], 16) / 100))
                else:
                    yield (values_map[item[:2]], int(item[-8:], 16))

    @staticmethod
    def _iterate_data_legacy(
        iterable: Iterable, values_map: dict[str, str]
    ) -> Generator:
        """Iterate data from hex words for legacy printers."""
        for item in iterable:
            # first byte means kind of sensor, last 4 bytes means value
            if item[:2] in values_map:
                yield (
                    values_map[item[:2]],
                    round(int(item[6:8], 16) / int(item[8:10], 16) * 100),
                )

    @staticmethod
    def _cleanse_status(status: str) -> str:
        """Cleanse and format status."""
        return " ".join(status.split()).strip()



"""Constants for Brother Printer library."""
from typing import Final

ATTR_CHARSET: Final = "charset"
ATTR_COUNTERS: Final = "counters"
ATTR_FIRMWARE: Final = "firmware"
ATTR_MAINTENANCE: Final = "maintenance"
ATTR_MODEL: Final = "model"
ATTR_NEXTCARE: Final = "nextcare"
ATTR_PAGE_COUNT: Final = "page_counter"
ATTR_SERIAL: Final = "serial"
ATTR_STATUS: Final = "status"
ATTR_UPTIME: Final = "uptime"

VAL_BELT_REMAIN: Final = "belt_unit_remaining_life"
VAL_BELT_REMAIN_PAGES: Final = "belt_unit_remaining_pages"
VAL_BLACK_COUNT: Final = "black_counter"
VAL_BLACK_DRUM_COUNT: Final = "black_drum_counter"
VAL_BLACK_DRUM_REMAIN: Final = "black_drum_remaining_life"
VAL_BLACK_DRUM_REMAIN_PAGES: Final = "black_drum_remaining_pages"
VAL_BLACK_INK: Final = "black_ink"
VAL_BLACK_INK_REMAIN: Final = "black_ink_remaining"
VAL_BLACK_INK_STATUS: Final = "black_ink_status"
VAL_BLACK_TONER: Final = "black_toner"
VAL_BLACK_TONER_REMAIN: Final = "black_toner_remaining"
VAL_BLACK_TONER_STATUS: Final = "black_toner_status"
VAL_BW_COUNT: Final = "bw_counter"
VAL_COLOR_COUNT: Final = "color_counter"
VAL_CYAN_COUNT: Final = "cyan_counter"
VAL_CYAN_DRUM_COUNT: Final = "cyan_drum_counter"
VAL_CYAN_DRUM_REMAIN: Final = "cyan_drum_remaining_life"
VAL_CYAN_DRUM_REMAIN_PAGES: Final = "cyan_drum_remaining_pages"
VAL_CYAN_INK: Final = "cyan_ink"
VAL_CYAN_INK_REMAIN: Final = "cyan_ink_remaining"
VAL_CYAN_INK_STATUS: Final = "cyan_ink_status"
VAL_CYAN_TONER: Final = "cyan_toner"
VAL_CYAN_TONER_REMAIN: Final = "cyan_toner_remaining"
VAL_CYAN_TONER_STATUS: Final = "cyan_toner_status"
VAL_DRUM_COUNT: Final = "drum_counter"
VAL_DRUM_REMAIN: Final = "drum_remaining_life"
VAL_DRUM_REMAIN_PAGES: Final = "drum_remaining_pages"
VAL_DRUM_STATUS: Final = "drum_status"
VAL_DUPLEX_COUNT: Final = "duplex_unit_pages_counter"
VAL_FUSER_REMAIN: Final = "fuser_remaining_life"
VAL_FUSER_REMAIN_PAGES: Final = "fuser_unit_remaining_pages"
VAL_IMAGE_COUNT: Final = "image_counter"
VAL_LASER_REMAIN: Final = "laser_remaining_life"
VAL_LASER_REMAIN_PAGES: Final = "laser_unit_remaining_pages"
VAL_MAGENTA_COUNT: Final = "magenta_counter"
VAL_MAGENTA_DRUM_COUNT: Final = "magenta_drum_counter"
VAL_MAGENTA_DRUM_REMAIN: Final = "magenta_drum_remaining_life"
VAL_MAGENTA_DRUM_REMAIN_PAGES: Final = "magenta_drum_remaining_pages"
VAL_MAGENTA_INK: Final = "magenta_ink"
VAL_MAGENTA_INK_REMAIN: Final = "magenta_ink_remaining"
VAL_MAGENTA_INK_STATUS: Final = "magenta_ink_status"
VAL_MAGENTA_TONER: Final = "magenta_toner"
VAL_MAGENTA_TONER_REMAIN: Final = "magenta_toner_remaining"
VAL_MAGENTA_TONER_STATUS: Final = "magenta_toner_status"
VAL_PAGE_COUNT: Final = "page_counter"
VAL_PF_1_REMAIN: Final = "pf_kit_1_remaining_life"
VAL_PF_1_REMAIN_PAGES: Final = "pf_kit_1_remaining_pages"
VAL_PF_MP_REMAIN: Final = "pf_kit_mp_remaining_life"
VAL_PF_MP_REMAIN_PAGES: Final = "pf_kit_mp_remaining_pages"
VAL_YELLOW_COUNT: Final = "yellow_counter"
VAL_YELLOW_DRUM_COUNT: Final = "yellow_drum_counter"
VAL_YELLOW_DRUM_REMAIN: Final = "yellow_drum_remaining_life"
VAL_YELLOW_DRUM_REMAIN_PAGES: Final = "yellow_drum_remaining_pages"
VAL_YELLOW_INK: Final = "yellow_ink"
VAL_YELLOW_INK_REMAIN: Final = "yellow_ink_remaining"
VAL_YELLOW_INK_STATUS: Final = "yellow_ink_status"
VAL_YELLOW_TONER: Final = "yellow_toner"
VAL_YELLOW_TONER_REMAIN: Final = "yellow_toner_remaining"
VAL_YELLOW_TONER_STATUS: Final = "yellow_toner_status"

CHARSET_MAP: Final = {
    "5": "latin2",
    "2004": "roman8",
    "8": "cyrillic",
    "12": "latin5",
}

PRINTER_TYPES: Final = ("ink", "laser")

OIDS: Final = {
    ATTR_CHARSET: "1.3.6.1.2.1.43.7.1.1.4.1.1",
    ATTR_COUNTERS: "1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.10.0",
    ATTR_FIRMWARE: "1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.17.0",
    ATTR_MAINTENANCE: "1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.8.0",
    ATTR_MODEL: "1.3.6.1.4.1.2435.2.3.9.1.1.7.0",
    ATTR_NEXTCARE: "1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.11.0",
    ATTR_PAGE_COUNT: "1.3.6.1.2.1.43.10.2.1.4.1.1",
    ATTR_SERIAL: "1.3.6.1.4.1.2435.2.3.9.4.2.1.5.5.1.0",
    ATTR_STATUS: "1.3.6.1.4.1.2435.2.3.9.4.2.1.5.4.5.2.0",
    ATTR_UPTIME: "1.3.6.1.2.1.1.3.0",
}

VALUES_COUNTERS: Final = {
    "00": VAL_PAGE_COUNT,
    "01": VAL_BW_COUNT,
    "02": VAL_COLOR_COUNT,
    "06": VAL_DUPLEX_COUNT,
    "12": VAL_BLACK_COUNT,
    "13": VAL_CYAN_COUNT,
    "14": VAL_MAGENTA_COUNT,
    "15": VAL_YELLOW_COUNT,
    "16": VAL_IMAGE_COUNT,
}

VALUES_LASER_MAINTENANCE: Final = {
    "11": VAL_DRUM_COUNT,
    "31": VAL_BLACK_TONER_STATUS,
    "32": VAL_CYAN_TONER_STATUS,
    "33": VAL_MAGENTA_TONER_STATUS,
    "34": VAL_YELLOW_TONER_STATUS,
    "41": VAL_DRUM_REMAIN,
    "63": VAL_DRUM_STATUS,
    "69": VAL_BELT_REMAIN,
    "6a": VAL_FUSER_REMAIN,
    "6b": VAL_LASER_REMAIN,
    "6c": VAL_PF_MP_REMAIN,
    "6d": VAL_PF_1_REMAIN,
    "6f": VAL_BLACK_TONER_REMAIN,
    "70": VAL_CYAN_TONER_REMAIN,
    "71": VAL_MAGENTA_TONER_REMAIN,
    "72": VAL_YELLOW_TONER_REMAIN,
    "73": VAL_CYAN_DRUM_COUNT,
    "74": VAL_MAGENTA_DRUM_COUNT,
    "75": VAL_YELLOW_DRUM_COUNT,
    "7e": VAL_BLACK_DRUM_COUNT,
    "79": VAL_CYAN_DRUM_REMAIN,
    "7a": VAL_MAGENTA_DRUM_REMAIN,
    "7b": VAL_YELLOW_DRUM_REMAIN,
    "80": VAL_BLACK_DRUM_REMAIN,
    "81": VAL_BLACK_TONER,
    "82": VAL_CYAN_TONER,
    "83": VAL_MAGENTA_TONER,
    "84": VAL_YELLOW_TONER,
    "a1": VAL_BLACK_TONER_REMAIN,
    "a2": VAL_CYAN_TONER_REMAIN,
    "a3": VAL_MAGENTA_TONER_REMAIN,
    "a4": VAL_YELLOW_TONER_REMAIN,
}

VALUES_INK_MAINTENANCE: Final = {
    "31": VAL_BLACK_INK_STATUS,
    "32": VAL_CYAN_INK_STATUS,
    "33": VAL_MAGENTA_INK_STATUS,
    "34": VAL_YELLOW_INK_STATUS,
    "6f": VAL_BLACK_INK_REMAIN,
    "70": VAL_CYAN_INK_REMAIN,
    "71": VAL_MAGENTA_INK_REMAIN,
    "72": VAL_YELLOW_INK_REMAIN,
    "81": VAL_BLACK_INK,
    "82": VAL_CYAN_INK,
    "83": VAL_MAGENTA_INK,
    "84": VAL_YELLOW_INK,
    "a1": VAL_BLACK_INK_REMAIN,
    "a2": VAL_CYAN_INK_REMAIN,
    "a3": VAL_MAGENTA_INK_REMAIN,
    "a4": VAL_YELLOW_INK_REMAIN,
}

VALUES_LASER_NEXTCARE: Final = {
    "73": VAL_LASER_REMAIN_PAGES,
    "77": VAL_PF_1_REMAIN_PAGES,
    "82": VAL_DRUM_REMAIN_PAGES,
    "86": VAL_PF_MP_REMAIN_PAGES,
    "88": VAL_BELT_REMAIN_PAGES,
    "89": VAL_FUSER_REMAIN_PAGES,
    "a4": VAL_BLACK_DRUM_REMAIN_PAGES,
    "a5": VAL_CYAN_DRUM_REMAIN_PAGES,
    "a6": VAL_MAGENTA_DRUM_REMAIN_PAGES,
    "a7": VAL_YELLOW_DRUM_REMAIN_PAGES,
}

PERCENT_VALUES: Final = (
    VAL_BELT_REMAIN,
    VAL_BLACK_DRUM_REMAIN,
    VAL_BLACK_INK_REMAIN,
    VAL_BLACK_TONER_REMAIN,
    VAL_CYAN_DRUM_REMAIN,
    VAL_CYAN_INK_REMAIN,
    VAL_CYAN_TONER_REMAIN,
    VAL_DRUM_REMAIN,
    VAL_FUSER_REMAIN,
    VAL_LASER_REMAIN,
    VAL_MAGENTA_DRUM_REMAIN,
    VAL_MAGENTA_INK_REMAIN,
    VAL_MAGENTA_TONER_REMAIN,
    VAL_PF_1_REMAIN,
    VAL_PF_MP_REMAIN,
    VAL_YELLOW_DRUM_REMAIN,
    VAL_YELLOW_INK_REMAIN,
    VAL_YELLOW_TONER_REMAIN,
)

OIDS_HEX: Final = (
    OIDS[ATTR_COUNTERS],
    OIDS[ATTR_MAINTENANCE],
    OIDS[ATTR_NEXTCARE],
)

UNSUPPORTED_MODELS: Final = ("mfc-8660dn", "mfc-8860dn")

UPTIME_FLUCTUATION_SECONDS: Final = 5



"""Brother exceptions."""


class BrotherError(Exception):
    """Base class for brother errors."""

    def __init__(self, status: str) -> None:
        """Initialize."""
        super().__init__(status)
        self.status = status


class SnmpError(BrotherError):
    """Raised when SNMP request ended in error."""


class UnsupportedModelError(BrotherError):
    """Raised when no model, serial no, firmware data."""



"""Setup file for brother module."""
from pathlib import Path

from setuptools import setup

PROJECT_DIR = Path(__file__).parent.resolve()
README_FILE = PROJECT_DIR / "README.md"
VERSION = "2.3.0"

with open("requirements.txt", encoding="utf-8") as file:
    requirements = file.read().splitlines()

setup(
    name="brother",
    version=VERSION,
    author="Maciej Bieniek",
    description=(
        "Python wrapper for getting data from Brother laser and inkjet "
        "printers via SNMP."
    ),
    long_description=README_FILE.read_text(encoding="utf-8"),
    long_description_content_type="text/markdown",
    url="https://github.com/bieniu/brother",
    license="Apache-2.0 License",
    packages=["brother"],
    package_data={"brother": ["py.typed"]},
    zip_safe=True,
    platforms="any",
    python_requires=">=3.10",
    install_requires=requirements,
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3 :: Only",
        "Typing :: Typed",
    ],
)
